-------------------------------------------------------------------------------
--                                                                           --
-- File         : gtcntl_uvw.vhd                                             --
--                                                                           --
-- Author(s)    : Dr. Lars Larsson (LL), www.trinamic.com                    --
--                                                                           --
-- Contributors :                                                            --
--                                                                           --
-- Description  : TMC60x commutator 'gtcntl_uvw' block (from schematic)      --
--                                                                           --
-- Creation     : LL, July 24, 2008                                          --
--                                                                           --
-- History      : LL, February 6, 2010 : BSD license added                   --
--                                                                           --
-------------------------------------------------------------------------------
-- Hint(s)      :                                                            --
-------------------------------------------------------------------------------
--
-- Copyright (c) 2010, TRINAMIC Motion Control GmbH & Co. KG 
-- All rights reserved.
--
-- Redistribution and use in source and binary forms, with or without 
-- modification, are permitted provided that the following conditions 
-- are met:
--
-- Redistributions of source code must retain the above copyright notice, 
-- this list of conditions and the following disclaimer. Redistributions 
-- in binary form must reproduce the above copyright notice, this list of 
-- conditions and the following disclaimer in the documentation and/or 
-- other materials provided with the distribution. 
--
-- Neither the name of the TRINAMIC nor the names of its contributors may 
-- be used to endorse or promote products derived from this software without 
-- specific prior written permission. 
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
-- THE POSSIBILITY OF SUCH DAMAGE.
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

entity gtcntl_uvw is 
  port ( 
          W, WZ :  in std_logic;
         WH, WL : out std_logic;
          V, VZ :  in std_logic;
         VH, VL : out std_logic;
          U, UZ :  in std_logic;
         UH, UL : out std_logic);
end gtcntl_uvw;

architecture behavior of gtcntl_uvw is

  signal wz_w : std_logic_vector ( 1 downto 0 ); 
  signal vz_v : std_logic_vector ( 1 downto 0 ); 
  signal uz_u : std_logic_vector ( 1 downto 0 );   

begin

  wz_w <= WZ & W; 
  vz_v <= VZ & V; 
  uz_u <= UZ & U; 
  
  with wz_w select WH <= '1' when "01", '0' when others; 
  with wz_w select WL <= '1' when "00", '0' when others; 
                                          
  with vz_v select VH <= '1' when "01", '0' when others; 
  with vz_v select VL <= '1' when "00", '0' when others; 
                                          
  with uz_u select UH <= '1' when "01", '0' when others; 
  with uz_u select UL <= '1' when "00", '0' when others; 
  
end behavior; 

---------------------------------------------------------------------------------------
