-------------------------------------------------------------------------------
--                                                                           --
-- File         : hall2uvw.vhd                                               --
--                                                                           --
-- Author(s)    : Dr. Lars Larsson (LL), www.trinamic.com                    --
--                                                                           --
-- Contributors :                                                            --
--                                                                           --
-- Description  : TMC60x commutator 'hall2uvw' block (from schematic)        --
--                                                                           --
-- Creation     : LL, July 24, 2008                                          --
--                                                                           --
-- History      : LL, February 6, 2010 : BSL license added                   --                                                          
--                                                                           --
-------------------------------------------------------------------------------
-- Hint(s)      :                                                            --
-------------------------------------------------------------------------------
--
-- Copyright (c) 2010, TRINAMIC Motion Control GmbH & Co. KG 
-- All rights reserved.
--
-- Redistribution and use in source and binary forms, with or without 
-- modification, are permitted provided that the following conditions 
-- are met:
--
-- Redistributions of source code must retain the above copyright notice, 
-- this list of conditions and the following disclaimer. Redistributions 
-- in binary form must reproduce the above copyright notice, this list of 
-- conditions and the following disclaimer in the documentation and/or 
-- other materials provided with the distribution. 
--
-- Neither the name of the TRINAMIC nor the names of its contributors may 
-- be used to endorse or promote products derived from this software without 
-- specific prior written permission. 
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
-- THE POSSIBILITY OF SUCH DAMAGE.
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

entity hall2uvw is 
  port ( 
         H3, H2, H1            :  in std_logic;
         W, WZ, V, VZ, U, UZ   : out std_logic);
end hall2uvw;

architecture behavior of hall2uvw is

  signal h321 : std_logic_vector ( 2 downto 0 ); 

begin

  h321 <= H3 & H2 & H1; 

    W  <= '1' when (h321="100" or h321="110") else '0'; 
    WZ <= '1' when (h321="010" or h321="101") else '0'; 
                                                      
    V  <= '1' when (h321="010" or h321="011") else '0'; 
    VZ <= '1' when (h321="001" or h321="110") else '0'; 
                                                      
    U  <= '1' when (h321="001" or h321="101") else '0'; 
    UZ <= '1' when (h321="011" or h321="100") else '0';                                    
                                                              
end behavior; 

-------------------------------------------------------------------------------
